<?php
/**
 * AJAX
 * 
 * @package 	TP Framework
 * @subpackage 	tp-framework\includes\classes
 * @since 		1.0.0
 * @author 		ThemePlugger <themeplugger@gmail.com>
 * @link 		http://themeplugger.com
 * @license 	GNU General Public License, version 3
 * @copyright 	© 2014 ThemePlugger
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'TP_Ajax' ) ) :

/**
 * TP_AJAX
 * 
 * @todo Add enqueue_scripts() method
 * @todo Add localize_script() method
 * 
 * @since 1.0.0
 */
class TP_Ajax {
	
	/** Constants **************************************************/
	
	/**
	 * Success status
	 * 
	 * @var string
	 */
	const SUCCESS_STATUS = 'success';
	
	/**
	 * Error status
	 * 
	 * @var string
	 */
	const ERROR_STATUS = 'error';
	
	/**
	 * Failed status
	 * 
	 * @var string
	 */
	const FAILED_STATUS = 'failed';
	
	
	/** Variables **************************************************/
	
	/**
	 * Response arguments
	 * 
	 * @since 1.0.0
	 * 
	 * @access private
	 * 
	 * @var array
	 */
	private $response = array();
	
	/**
	 * Ajax callback
	 * 
	 * @since 1.0.0
	 * 
	 * @var mixed
	 */
	public $ajax;
	
	/**
	 * Ajax ID
	 * 
	 * @since 1.0.0
	 * 
	 * @var string
	 */
	public $ajax_id;
	
	/**
	 * Ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @access private
	 * 
	 * @var string
	 */
	private $ajax_nonce;
	
	/**
	 * Unauthenticated ajax callback
	 * 
	 * @since 1.0.0
	 * 
	 * @var mixed
	 */
	public $ajax_nopriv;
	
	/**
	 * Unauthenticated ajax ID
	 * 
	 * @since 1.0.0
	 * 
	 * @var string
	 */
	public $ajax_nopriv_id;
	
	/**
	 * Unauthenticated ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @access private
	 * 
	 * @var string
	 */
	private $ajax_nopriv_nonce;
	
	
	/** Magic methods **************************************************/
	
	/**
	 * Constructor
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	array $args
	 * @return 	void
	 */
	public function __construct( $args = array() ) {
		// Utilize WordPress' pluggable functions
        if ( ! function_exists( 'wp_create_nonce' ) ) {
            include( ABSPATH . "wp-includes/pluggable.php" );
        }
				
		if ( sizeof( $args ) > 0 ) {
			// Assign ajax callback function
			if ( isset( $args['ajax'] ) ) {
				$this->ajax = $args['ajax'];
				
				// Assign a unique ID for authenticated ajax
				if ( isset( $args['ajax_id'] ) ) {
					$this->ajax_id = $args['ajax_id'];
				} else {
					$this->ajax_id = is_array( $args['ajax'] ) ? $args['ajax'][0] : $args['ajax'];
				}
			}
			
			// Assign unauthenticated ajax callback function
			if ( isset( $args['ajax_nopriv'] ) ) {
				$this->ajax_nopriv = $args['ajax_nopriv'];
				
				// Assign a unique ID for unauthenticated ajax
				if ( isset( $args['ajax_nopriv_id'] ) ) {
					$this->ajax_nopriv_id = $args['ajax_nopriv_id'];
				} else {
					$this->ajax_nopriv_id = is_array( $args['ajax_nopriv'] ) ? $args['ajax_nopriv'][0] : $args['ajax_nopriv'];
				}
			}
			
			// Use predefined self methods to handle both
			// authenticated and unauthenticated ajax callback functions.
			// This is triggered when this class is extended and an interface
			// "TP_Interface_Ajax" is being implemented.
			if ( method_exists( $this, 'init' ) ) {
				$this->ajax 		= array( $this, 'ajax' 			);
				$this->ajax_nopriv 	= array( $this, 'ajax_nopriv' 	);
			}
			
			// Set ajax nonce
			$this->set_nonce();
			
			// Set unauthenticated nonce
			$this->set_nopriv_nonce();
		}
		
		// Default response arguments
		$this->response = array(
			'status' 	=> '', // success|error|failed
			'message' 	=> ''
		);
	}
	
	
	/** Common methods **************************************************/
	
	/**
	 * Retrieve response
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	bool $echo
	 * @return 	void|array
	 */
	public function get_response( $echo = false ) {
		if ( $echo == true ) {
			echo json_encode( $this->response );
		} else {
			return $this->response;
		}
	}
	
	
	/** Ajax methods **************************************************/
	
	/**
	 * Generate ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @uses wp_create_nonce()
	 * 
	 * @return string
	 */
	public function set_nonce() {
		$this->ajax_nonce = wp_create_nonce( $this->ajax_id );
	}
	
	/**
	 * Retrieve ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @return string
	 */
	public function get_nonce() {
		return $this->ajax_nonce;
	}
	
	/**
	 * Retrieve ajax nonce field
	 * 
	 * @since 1.0.0
	 * 
	 * @uses wp_nonce_field()
	 * 
	 * @param 	bool 		$referer
	 * @param 	bool 		$echo
	 * @return 	void|string $nonce_field
	 */
	public function get_nonce_field( $referer = true, $echo = true ) {
		$nonce_field = wp_nonce_field( $this->ajax_id, $this->ajax_id, $referer, false );
		
		if ( $echo == true ) {
			echo $nonce_field;
		} else {
			return $nonce_field;
		}
	}
	
	/**
	 * Verify ajax
	 * 
	 * @since 1.0.0
	 */
	public function verify() {
		if ( ! $this->verify_nonce() ) {
			$this->set_status( self::ERROR_STATUS );
			$this->set_message( __( 'Something went wrong.', 'tp_framework' ) );
		}
		
		if ( $this->response['status'] == self::ERROR_STATUS ) {
			$this->get_response( true );
			exit();
		}
	}
	
	/**
	 * Verify ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @uses wp_verify_nonce()
	 * 
	 * @return bool
	 */
	public function verify_nonce() {
		return isset( $_POST[$this->ajax_id] ) && wp_verify_nonce( $_POST[$this->ajax_id], $this->ajax_id );
	}
	
	
	/** Unauthenticated ajax methods **************************************************/
	
	/**
	 * Generate unauthenticated ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @uses wp_create_nonce()
	 * 
	 * @return string
	 */
	public function set_nopriv_nonce() {
		$this->ajax_nopriv_nonce = wp_create_nonce( $this->ajax_nopriv_id );
	}
	
	/**
	 * Retrieve unauthenticated ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @return string
	 */
	public function get_nopriv_nonce() {
		return $this->ajax_nopriv_nonce;
	}
	
	/**
	 * Retrieve unauthenticated ajax nonce field
	 * 
	 * @since 1.0.0
	 * 
	 * @uses wp_nonce_field()
	 * 
	 * @param 	bool 		$referer
	 * @param 	bool 		$echo
	 * @return 	void|string $nonce_field
	 */
	public function get_nopriv_nonce_field( $referer = true, $echo = true ) {
		$nonce_field = wp_nonce_field( $this->ajax_nopriv_id, $this->ajax_nopriv_id, $referer, false );
		
		if ( $echo == true ) {
			echo $nonce_field;
		} else {
			return $nonce_field;
		}
	}
	
	/**
	 * Verify unauthenticated ajax
	 * 
	 * @since 1.0.0
	 */
	public function verify_nopriv() {
		if ( ! $this->verify_nopriv_nonce() ) {
			$this->set_status( self::ERROR_STATUS );
			$this->set_message( __( 'Something went wrong.', 'tp_framework' ) );
		}
		
		if ( $this->response['status'] == self::ERROR_STATUS ) {
			$this->get_response( true );
			exit();
		}
	}
	
	/**
	 * Verify unathenticated ajax nonce
	 * 
	 * @since 1.0.0
	 * 
	 * @uses wp_verify_nonce()
	 * 
	 * @return bool
	 */
	public function verify_nopriv_nonce() {
		return isset( $_POST[$this->ajax_nopriv_id] ) && wp_verify_nonce( $_POST[$this->ajax_nopriv_id], $this->ajax_nopriv_id );
	}
	
	
	/** Setup methods **************************************************/
	
	/**
	 * Setup callbacks
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	int 	$priority
	 * @param 	int 	$accepted_args
	 * @return 	void
	 */
	public function setup_callbacks( $priority = 10, $accepted_args = 1 ) {
		self::setup_ajax( $this->ajax_id, $this->ajax, $priority, $accepted_args );
		self::setup_ajax_nopriv( $this->ajax_nopriv_id, $this->ajax_nopriv, $priority, $accepted_args );
	}
	
	/**
	 * Add an action callback for an authenticated ajax
	 * 
	 * @uses add_action()
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	string 	$ajax 			Unique ajax ID
	 * @param 	string 	$function 		Callback function
	 * @param 	int 	$priority 		Priority
	 * @param 	int 	$accepted_args 	Accepted arguments
	 * @return 	void
	 */
	public static function setup_ajax( $ajax, $function, $priority = 10, $accepted_args = 1 ) {
		add_action( "wp_ajax_{$ajax}", $function, $priority, $accepted_args );
	}
	
	/**
	 * Add an action callback for an unauthenticated ajax
	 * 
	 * @since 1.0.0
	 * 
	 * @uses add_action()
	 * 
	 * @param 	string 	$ajax_nopriv 	Unique ajax ID
	 * @param 	string 	$function 		Callback function
	 * @param 	int 	$priority 		Priority
	 * @param 	int 	$accepted_args 	Accepted arguments
	 * @return 	void
	 */
	public static function setup_ajax_nopriv( $ajax_nopriv, $function, $priority = 10, $accepted_args = 1 ) {
		add_action( "wp_ajax_nopriv_{$ajax_nopriv}", $function, $priority, $accepted_args );
	}
	
	
	/** Set methods **************************************************/
	
	/**
	 * Set response property key
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	string 	$property
	 * @param 	mixed 	$value
	 * @return 	void
	 */
	public function set_response( $property, $value ) {
		$this->response[$property] = $value;
	}
	
	/**
	 * Set response status
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	string $status
	 * @return 	void
	 */
	public function set_status( $status ) {
		$this->set_response( 'status', $status );
	}
	
	/**
	 * Set response message
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	string $message
	 * @return 	void
	 */
	public function set_message( $message ) {
		$this->set_response( 'message', $message );
	}
}

endif;