<?php
/**
 * Autoload
 * 
 * Handles initialization of classes needed.
 * 
 * @package 	TP Framework
 * @subpackage 	tp-framework\includes\classes
 * @since 		1.0.0
 * @author 		ThemePlugger <themeplugger@gmail.com>
 * @link 		http://themeplugger.com
 * @license 	GNU General Public License, version 3
 * @copyright 	© 2014 ThemePlugger
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'TP_Autoload' ) ) :

/**
 * TP Autoload
 * 
 * @since 1.0.0
 */
class TP_Autoload {
	
	/**
	 * Class prefix
	 * 
	 * @since 1.0.0
	 * 
	 * @var string
	 */
	private $prefix;
	
	/**
	 * Base directory path
	 * 
	 * @since 1.0.0
	 * 
	 * @var string
	 */
	private $base_dir;
	
	/**
	 * Arguments
	 * 
	 * @since 1.0.0
	 * 
	 * @var array
	 */
	private $args = array();
	
	
	/**
	 * Constructor
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	string 				$prefix
	 * @param 	string 	Optional 	$base_dir
	 * @param 	array 	Optional 	$args
	 */
	protected function __construct( $prefix, $base_dir = '', $args = array() ) {
		$this->prefix 	= $prefix;
		$this->base_dir = $base_dir;
		$this->args 	= $args;
		
		if ( empty( $this->base_dir ) ) {
			$this->base_dir = dirname( __FILE__ );
		}
		
		$this->base_dir = tp_trailingslashit( $this->base_dir );
	}
	
	/**
	 * Register autoload
	 * 
	 * @since 1.0.0
	 * 
	 * @param 	string 				$prefix
	 * @param 	string 	Optional 	$base_dir
	 * @param 	array 	Optional 	$args
	 */
	public static function register( $prefix, $base_dir = '', $args = array() ) {
		$loader = new self( $prefix, $base_dir, $args );
		
		// Register autoload callback
		spl_autoload_register( array( $loader, '__autoload' ) );
	}
	
	/**
	 * Autoload callback
	 * 
	 * @since 1.0.0
	 * 
	 * @param string $class Class name
	 * @return void
	 */
	public function __autoload( $class ) {
		if ( substr( $class, 0, strlen( $this->get_prefix() ) ) == $this->get_prefix() ) {
			$file = $this->get_file( $class );
			
			if ( is_file( $file ) ) {
				include $file;
			}
		}
	}
	
	/**
	 * Retrieve filename
	 * 
	 * @since 1.0.0
	 * 
	 * @param string $class
	 * @return string $filename
	 */
	private function get_file( $class ) {
		return $this->sanitize_filename( $class );
	}
	
	/**
	 * Sanitize filename
	 * 
	 * @since 1.0.0
	 * 
	 * @param string 	$class
	 * @return string 	$filename
	 */
	private function sanitize_filename( $class ) {
		// Remove prefix
		$filename = str_replace( $this->get_prefix(), '', $class );
		
		// Replace "underscores" with "spaces"
		$filename = str_replace( '_', ' ', $filename );
		
		// Lowercase all letters
		$filename = strtolower( $filename );
		
		// Replace "underscores" with "dashes"
		$filename = str_replace( ' ', '-', $filename );
		
		// Construct file path
		$filename = $this->get_base_dir() . "{$filename}.php";
		
		return $filename;
	}
	
	/**
	 * Retrieve class prefix
	 * 
	 * @since 1.0.0
	 * 
	 * @return string
	 */
	private function get_prefix() {
		return $this->prefix;
	}
	
	/**
	 * Retrieve base directory path
	 * 
	 * @since 1.0.0
	 * 
	 * @return string
	 */
	private function get_base_dir() {
		return $this->base_dir;
	}
}

endif;
