<?php
/**
 * Plugin base class
 * 
 * @package 	TP Framework
 * @subpackage 	tp-framework\includes\classes
 * @since 		1.0.0
 * @author 		ThemePlugger <themeplugger@gmail.com>
 * @link 		http://themeplugger.com
 * @license 	GNU General Public License, version 3
 * @copyright 	© 2014 ThemePlugger
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'TP_Base_Plugin' ) ) :

/**
 * TP Base Plugin
 * 
 * @todo Test loadt_textdomain()
 * @todo Test WPMU
 * 
 * @since 1.0.0
 * 
 * @abstract
 */
abstract class TP_Base_Plugin extends TP_Base {
	
	/**
	 * Registers plugin activation hook
	 * 
	 * @since 1.0.0
	 * 
	 * @uses register_activation_hook()
	 * 
	 * @param string 		$file
	 * @param string|array 	$function 	array( $this, 'activate' )
	 */
	protected function setup_activation( $file = '', $function = '' ) {
		register_activation_hook( $file ? $file : $this->file, $function ? $function : array( $this, 'activate' ) );
	}
	
	/**
	 * Registers plugin deactivation hook
	 * 
	 * @since 1.0.0
	 * 
	 * @uses register_deactivation_hook()
	 * 
	 * @param string 		$file
	 * @param string|array 	$function 	array( $this, 'deactivate' )
	 */
	protected function setup_deactivation( $file = '', $function = '' ) {
		register_deactivation_hook( $file ? $file : $this->file, $function ? $function : array( $this, 'deactivate' ) );
	}
	
	/**
	 * Sets up WPMU-environment
	 * 
	 * @since 1.0.0
	 */
	protected function setup_wpmu() {
		add_action( 'wpmu_new_blog', array( $this, 'activate_new_site' ) );
	}
	
	
	/** Hook methods **************************************************/
	
	/**
	 * Load text domain for translation
	 * 
	 * @since 1.0.0
	 */
	public function load_textdomain() {
		
		// Traditional WordPress plugin locale filter
		$locale        = apply_filters( 'plugin_locale', get_locale(), $this->textdomain );
		$mofile        = sprintf( '%1$s-%2$s.mo', $this->textdomain, $locale );

		// Setup paths to current locale file
		$mofile_local  = $this->languages_dir . $mofile;
		$mofile_global = tp_trailingslashit( WP_LANG_DIR . DIRECTORY_SEPARATOR . "{$this->id}" ) . $mofile;

		// Look in global /wp-content/languages/{id} folder
		load_textdomain( $this->textdomain, $mofile_global );

		// Look in local /wp-content/plugins/{id}/languages/ folder
		load_textdomain( $this->textdomain, $mofile_local );

		// Look in global /wp-content/languages/plugins/
		load_plugin_textdomain( $this->textdomain );
				
		// load_plugin_textdomain( $this->plugin_textdomain, false, $this->languages_dir );
	}
	
	/**
	 * Fired when the plugin is activated.
	 * 
	 * @since 1.0.0
	 * 
	 * @param boolean $network_wide True if WPMU super admin uses 
	 * 								"Network Activate" action, false if WPMU is disabled 
	 * 								or plugin is activated on an individual blog.
	 */
	public function activate( $network_wide = '' ) {
		if ( function_exists( 'is_multisite' ) && is_multisite() && $network_wide ) {
			// Get all blog ids
			$blog_ids = self::get_blog_ids();
			
			foreach ( $blog_ids as $blog_id ) {
				switch_to_blog( $blog_id );
				
				$this->single_activate();
				
				restore_current_blog();
			}
		} else {
			$this->single_activate();
		}
	}
	
	/**
	 * Fired when the plugin is deactivated.
	 * 
	 * @since 1.0.0
	 * 
	 * @param boolean $network_wide True if WPMU super admin uses
	 * 								"Network Activate" action, false if WPMU is disabled 
	 * 								or plugin is activated on an individual blog.
	 */
	public function deactivate( $network_wide = '' ) {
		if ( function_exists( 'is_multisite' ) && is_multisite() && $network_wide ) {
			// Get all blog ids
			$blog_ids = self::get_blog_ids();
			
			foreach ( $blog_ids as $blog_id ) {
				switch_to_blog( $blog_id );
				
				$this->single_deactivate();
				
				restore_current_blog();
			}
		} else {
			$this->single_deactivate();
		}
	}
	
	/**
	 * Fired for each blog when the plugin is activated.
	 * 
	 * @since 1.0.0
	 */
	protected function single_activate() {
		do_action( $this->prefix( 'activate' ) );
	}
	
	
	/**
	 * Fired for each blog when the plugin is deactivated.
	 * 
	 * @since 1.0.0
	 */
	protected function single_deactivate() {
		do_action( $this->prefix( 'deactivate' ) );
	}
	
	/**
	 * Fired when a new site is activated with a WPMU environment.
	 * 
	 * @since 1.0.0
	 * 
	 * @param int $blog_id ID of the new blog.
	 */
	public function activate_new_site( $blog_id ) {
		if ( 1 !== did_action( 'wpmu_new_blog' ) ) {
			return;
		}
		
		switch_to_blog( $blog_id );
		
		$this->single_activate();
		
		do_action( $this->prefix( 'activate_new_site' ), $blog_id );
		
		restore_current_blog();
	}
	
	
	/** Helper methods **************************************************/
		
	/**
	 * Get all blog ids of blogs in the current network that are:
	 * - not archived
	 * - not spam
	 * - not deleted
	 * 
	 * @since 1.0
	 * 
	 * @return array|false The blog ids, false if no matches.
	 */
	protected static function get_blog_ids() {
		global $wpdb;
		
		$sql = "SELECT blog_id FROM $wpdb->blogs WHERE archived = '0' AND spam = '0' AND deleted = '0'";
			
		return $wpdb->get_col( $sql );
	}
}
	
endif;